﻿using EmitMapper;
using EmitMapper.MappingConfiguration;
using System;

namespace EmitMapperComponent
{
    internal class Program
    {
        private static void Main(string[] args)
        {
            var user = new UserInfo
            {
                id = 12,
                name = "张三",
                address = "北京",
                teacher = new Teacher
                {
                    id = 11,
                    name = "王五"
                }
            };

            var mapper = ObjectMapperManager.DefaultInstance.GetMapper<UserInfo, UserInfoDTO>(
                new DefaultMapConfig()
                .ConvertUsing<Teacher, string>(t => t.name) //如果是对象=》字段，默认转换对转换为对象的完整名。
                .MatchMembers((x, y) =>
                {
                    if (x == "teacher" && y == "teachername")
                    {
                        return true;
                    }
                    return x == y;
                })
                );
            UserInfoDTO userdto = mapper.Map(user);

            //UserInfoDTO userdto1 = new UserInfoDTO
            //{
            //    id = user.id,
            //    name = user.name,
            //    teachername = user.teacher != null ? user.teacher.name : "",
            //};
            Sourse src = new Sourse
            {
                A = 1,
                B = 0M,
                C = "2011/9/21 0:00:00",
                D = new Inner
                {
                    D2 = Guid.NewGuid()
                },
                E = "test"
            };
            //=>一、(默认映射配置器)属性名称一样的情况;类型可以不一样，但基元素类型转换可以进行ToString。（注意是属性名称是区分大小写）
            //=》写法一
            //ObjectsMapper<Sourse, Dest> mapper =
            //ObjectMapperManager.DefaultInstance.GetMapper<Sourse, Dest>();
            //Dest dst = mapper.Map(src);

            //=》简写二
            Dest dst = null;
            dst = ObjectMapperManager.DefaultInstance.GetMapper<Sourse, Dest>().Map(src, dst);

            //=》二、自定义映射配置器
            ObjectsMapper<Sourse, Dest> mapper1 =
                new ObjectMapperManager().GetMapper<Sourse, Dest>
                (
                 new DefaultMapConfig()
                .IgnoreMembers<Sourse, Dest>(new string[] { "A" })
                .NullSubstitution<decimal?, decimal>((value) => { return -1M; }) //如果不是语句块，则可以省略return;如果是加上return，则需要加上花括号 { return }
                //.NullSubstitution<decimal?, decimal>((value) =>  -1M)
                .ConvertUsing<Inner, Inner2>(value => new Inner2 { D12 = value.D1, D22 = value.D2 })
                //.ConvertUsing<Inner, Inner2>(value => { return new Inner2 { D12 = value.D1, D22 = value.D2 }; })
                //=>方法体的写法
                //.PostProcess<Dest>((value, state) =>
                //{
                //    value.F = "nothing";
                //    value.A = 111;
                //    return value;
                //})
                //);
                //=>return的写法
                //.PostProcess<Dest>((value, state) =>
                //{
                //    return new Dest
                //    {
                //        F = "nothing",
                //        A = 111,
                //    };
                //})
                //.PostProcess<Dest>((value, state) =>
                //    new Dest
                //    {
                //        F = "noting",
                //        A = 111,
                //    })
                //);
                .PostProcess<Dest>((Dest value, object state) =>
                    new Dest
                    {
                        F = "noting",
                        A = 111,
                    })
                );
            Dest dst1 = mapper1.Map(src);

            //=>其他
            //=>委托写法
            //EmitMapper.MappingConfiguration.MappingOperations.ValuesPostProcessor<Dest> ValuesPostProcessor1 = delegate(Dest dest1, object state) { return new Dest { }; };
            EmitMapper.MappingConfiguration.MappingOperations.ValuesPostProcessor<Dest> ValuesPostProcessor2 = (d, s) => { return new Dest { }; };
        }
    }
}